/*!
 *****************************************************************************
 * @file:
 * @brief:
 *-----------------------------------------------------------------------------
 *
Copyright (c) 2013-2018 Analog Devices, Inc. All Rights Reserved.

This software is proprietary to Analog Devices, Inc. and its licensors.
By using this software you agree to the terms of the associated
Analog Devices Software License Agreement.
 ******************************************************************************/

#include "ADuCM410.h"
#include "WdtLib.h"
#include "common.h"
#include "DioLib.h"
#include "ClkLib.h"
#include "UrtLib.h"  

#define UART_RX_BUFFER_LEN 100
#define UART_TX_BUFFER_LEN 8
//Variables declaration
extern uint8_t UartRxBuf[UART_RX_BUFFER_LEN];
extern uint32_t rxCnt;
extern uint32_t rxFifoCnt;
extern uint8_t UartTxBuf[UART_TX_BUFFER_LEN];
extern uint32_t txCnt;

// Function Definitions
void delay(uint32_t t);
void Uart0Test(void);
void Uart1Test(void);


uint8_t UartRxBuf[UART_RX_BUFFER_LEN];
uint32_t rxCnt = 0;
uint32_t rxFifoCnt = 0;
uint8_t UartTxBuf[UART_TX_BUFFER_LEN] = {'A','D','u','C','M','4','1','0'};
uint32_t txCnt = 0;


void Uart0Test()
{
   // Configure  UART0 pins
   DioCfgPin(pADI_GPIO0, PIN4, 2);
   DioCfgPin(pADI_GPIO0, PIN5, 2);
   
   DioPullCfgPin(pADI_GPIO0, PIN4, 1);
   DioPullCfgPin(pADI_GPIO0, PIN5, 1);
   DioPulPin(pADI_GPIO0, PIN4, 1);
   DioPulPin(pADI_GPIO0, PIN5, 1);
   
   UrtSetup(pADI_UART0,&gUrt0Setup);
   NVIC_EnableIRQ(UART0_IRQn);
}

void Uart1Test()
{
   // Configure  UART1 pins
   DioCfgPin(pADI_GPIO1, PIN0, 1);
   DioCfgPin(pADI_GPIO1, PIN1, 1);
   
   DioPullCfgPin(pADI_GPIO1, PIN0, 1);
   DioPullCfgPin(pADI_GPIO1, PIN1, 1);
   DioPulPin(pADI_GPIO1, PIN1, 1);
   DioPulPin(pADI_GPIO1, PIN0, 1);
   
   //OR
   //DioCfgPin(pADI_GPIO2, PIN4, 2);
   //DioCfgPin(pADI_GPIO2, PIN5, 2);
   
   UrtSetup(pADI_UART1,&gUrt1Setup);
   NVIC_EnableIRQ(UART1_IRQn);
}

int main (void)
{
   
   rxCnt = 0;
   txCnt = 0;
   rxFifoCnt = 0;
   WdtGo(false);
   
   ClkSetup(&gClkSetup);
   
    SystemInit();
   INIT();
   SafetyWait();
   
   Uart1Test();
   for(uint32_t i=0;i<UART_TX_BUFFER_LEN;i++)
   {
      //while(!(pADI_UART1->LSR&BITM_UART_LSR_THRE)); //wait for ready to write new data
      while(pADI_UART1->TFC>=16);//check 16 Bytes FIFO full or not 
      pADI_UART1->TX = UartTxBuf[i];   
   }
   
   while(1)
   {
      if(txCnt<rxCnt)
      {
         //while(!(pADI_UART0->LSR&BITM_UART_LSR_THRE)); //wait for ready to write new data
         //pADI_UART0->TX = UartRxBuf[txCnt++];     
         while(!(pADI_UART1->LSR&BITM_UART_LSR_THRE)); //wait for ready to write new data
         pADI_UART1->TX = UartRxBuf[txCnt++];
      }
      if(pADI_UART1->RFC>0)
      {
         for(uint32_t i=0;i<pADI_UART1->RFC;i++)
            UartRxBuf[rxCnt++] = (uint8_t)pADI_UART1->RX;
      }
   }

}

void delay(uint32_t t)
{
   uint32_t i;
   for(i=0;i<t;i++);
}

void UART0_Int_Handler()
{
   uint32_t sta,lsr;
   uint32_t iir = pADI_UART0->IIR;
   lsr = pADI_UART0->LSR;
   (void)lsr;
   //if(iir&BITM_UART_IIR_NIRQ)   //interrup flag
   //{
   sta = (iir&BITM_UART_IIR_STAT)>>BITP_UART_IIR_STAT;
   
   switch(sta)
   {
      //Modem Status Interrupt (Read MSR Register to Clear)
   case ENUM_UART_IIR_STAT_MODEMINT:  
      break;
      //Transmit Buffer Empty Interrupt (Write to Tx Register or Read IIR Register to clear)
   case ENUM_UART_IIR_STAT_TXEMPTYINT:
      break;
      //Receive Buffer Full Interrupt (Read Rx Register to Clear)
   case ENUM_UART_IIR_STAT_RXFULLINT:
      rxFifoCnt = pADI_UART0->RFC;
      for(uint32_t i=0;i<rxFifoCnt;i++)
         UartRxBuf[rxCnt++] = (uint8_t)pADI_UART0->RX;
      DEBUG_PIN1_TOGGLE();
      break;
      //Receive FIFO Time-out Interrupt (Read Rx Register to Clear)
   case ENUM_UART_IIR_STAT_RXTIMEOUTINT:
      break;
      //Receive Line Status Interrupt (Read LSR Register to Clear)
   case ENUM_UART_IIR_STAT_RXLINEINT:
      break;
   default: 
      break;
   }
   // }
   DEBUG_PIN0_TOGGLE();
}

void UART1_Int_Handler()
{
   uint32_t sta,lsr;
   uint32_t iir = pADI_UART1->IIR;
   lsr = pADI_UART1->LSR;
   (void)lsr;
   //if(iir&BITM_UART_IIR_NIRQ)   //interrup flag
   //{
   sta = (iir&BITM_UART_IIR_STAT)>>BITP_UART_IIR_STAT;
   
   switch(sta)
   {
      //Modem Status Interrupt (Read MSR Register to Clear)
   case ENUM_UART_IIR_STAT_MODEMINT:  
      break;
      //Transmit Buffer Empty Interrupt (Write to Tx Register or Read IIR Register to clear)
   case ENUM_UART_IIR_STAT_TXEMPTYINT:
      break;
      //Receive Buffer Full Interrupt (Read Rx Register to Clear), triggered by number setting in FCR
   case ENUM_UART_IIR_STAT_RXFULLINT:
      rxFifoCnt = pADI_UART1->RFC;
      for(uint32_t i=0;i<rxFifoCnt;i++)
         UartRxBuf[rxCnt++] = (uint8_t)pADI_UART1->RX;
      DEBUG_PIN1_TOGGLE();
      break;
      //Receive FIFO Time-out Interrupt (Read Rx Register to Clear)
   case ENUM_UART_IIR_STAT_RXTIMEOUTINT:
      break;
      //Receive Line Status Interrupt (Read LSR Register to Clear)
   case ENUM_UART_IIR_STAT_RXLINEINT:
      break;
   default: 
      break;
   }
   // }
   DEBUG_PIN0_TOGGLE();
   
}




